@@ -31,10 +31,7 @@
 #include "clutter-private.h"
 
 #include <X11/Xatom.h>
-
-#ifdef HAVE_XKB
 #include <X11/XKBlib.h>
-#endif
 
 typedef struct _ClutterKeymapX11Class   ClutterKeymapX11Class;
 typedef struct _DirectionCacheEntry     DirectionCacheEntry;
@@ -71,7 +68,6 @@ struct _ClutterKeymapX11
 
   PangoDirection current_direction;
 
-#ifdef HAVE_XKB
   XkbDescPtr xkb_desc;
   int xkb_event_base;
   guint xkb_map_serial;
@@ -79,7 +75,6 @@ struct _ClutterKeymapX11
   guint current_cache_serial;
   DirectionCacheEntry group_direction_cache[4];
   int current_group;
-#endif
 
   guint caps_lock_state : 1;
   guint num_lock_state  : 1;
@@ -110,8 +105,6 @@ G_DEFINE_TYPE_WITH_CODE (ClutterKeymapX11, clutter_keymap_x11, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_EVENT_TRANSLATOR,
                                                 clutter_event_translator_iface_init));
 
-#ifdef HAVE_XKB
-
 /* code adapted from gdk/x11/gdkkeys-x11.c - update_modmap */
 static void
 update_modmap (Display          *display,
@@ -214,9 +207,7 @@ get_xkb (ClutterKeymapX11 *keymap_x11)
 
   return keymap_x11->xkb_desc;
 }
-#endif /* HAVE_XKB */
 
-#ifdef HAVE_XKB
 static void
 update_locked_mods (ClutterKeymapX11 *keymap_x11,
                     gint              locked_mods)
@@ -242,9 +233,7 @@ update_locked_mods (ClutterKeymapX11 *keymap_x11,
     g_signal_emit_by_name (keymap_x11->backend, "key-lock-changed");
 #endif
 }
-#endif /* HAVE_XKB */
 
-#ifdef HAVE_XKB
 /* the code to retrieve the keymap direction and cache it
  * is taken from GDK:
  *      gdk/x11/gdkkeys-x11.c
@@ -345,13 +334,11 @@ get_direction_from_cache (ClutterKeymapX11 *keymap_x11,
 
   return direction;
 }
-#endif /* HAVE_XKB */
 
 static void
 update_direction (ClutterKeymapX11 *keymap_x11,
                   int               group)
 {
-#ifdef HAVE_XKB
   XkbDescPtr xkb = get_xkb (keymap_x11);
   Atom group_atom;
 
@@ -363,7 +350,6 @@ update_direction (ClutterKeymapX11 *keymap_x11,
       keymap_x11->current_group_atom = group_atom;
       keymap_x11->has_direction = TRUE;
     }
-#endif /* HAVE_XKB */
 }
 
 static void
@@ -371,54 +357,49 @@ clutter_keymap_x11_constructed (GObject *gobject)
 {
   ClutterKeymapX11 *keymap_x11 = CLUTTER_KEYMAP_X11 (gobject);
   ClutterBackendX11 *backend_x11;
+  gint xkb_major = XkbMajorVersion;
+  gint xkb_minor = XkbMinorVersion;
 
   g_assert (keymap_x11->backend != NULL);
   backend_x11 = CLUTTER_BACKEND_X11 (keymap_x11->backend);
 
-#ifdef HAVE_XKB
-  {
-    gint xkb_major = XkbMajorVersion;
-    gint xkb_minor = XkbMinorVersion;
+  if (XkbLibraryVersion (&xkb_major, &xkb_minor))
+    {
+      xkb_major = XkbMajorVersion;
+      xkb_minor = XkbMinorVersion;
 
-    if (XkbLibraryVersion (&xkb_major, &xkb_minor))
-      {
-        xkb_major = XkbMajorVersion;
-        xkb_minor = XkbMinorVersion;
+      if (XkbQueryExtension (backend_x11->xdpy,
+                             NULL,
+                             &keymap_x11->xkb_event_base,
+                             NULL,
+                             &xkb_major, &xkb_minor))
+        {
+          Bool detectable_autorepeat_supported;
 
-        if (XkbQueryExtension (backend_x11->xdpy,
-                               NULL,
-                               &keymap_x11->xkb_event_base,
-                               NULL,
-                               &xkb_major, &xkb_minor))
-          {
-            Bool detectable_autorepeat_supported;
+          backend_x11->use_xkb = TRUE;
 
-            backend_x11->use_xkb = TRUE;
+          XkbSelectEvents (backend_x11->xdpy,
+                           XkbUseCoreKbd,
+                           XkbNewKeyboardNotifyMask | XkbMapNotifyMask | XkbStateNotifyMask,
+                           XkbNewKeyboardNotifyMask | XkbMapNotifyMask | XkbStateNotifyMask);
 
-            XkbSelectEvents (backend_x11->xdpy,
-                             XkbUseCoreKbd,
-                             XkbNewKeyboardNotifyMask | XkbMapNotifyMask | XkbStateNotifyMask,
-                             XkbNewKeyboardNotifyMask | XkbMapNotifyMask | XkbStateNotifyMask);
+          XkbSelectEventDetails (backend_x11->xdpy,
+                                 XkbUseCoreKbd, XkbStateNotify,
+                                 XkbAllStateComponentsMask,
+                                 XkbGroupLockMask | XkbModifierLockMask);
 
-            XkbSelectEventDetails (backend_x11->xdpy,
-                                   XkbUseCoreKbd, XkbStateNotify,
-                                   XkbAllStateComponentsMask,
-                                   XkbGroupLockMask | XkbModifierLockMask);
+          /* enable XKB autorepeat */
+          XkbSetDetectableAutoRepeat (backend_x11->xdpy,
+                                      True,
+                                      &detectable_autorepeat_supported);
 
-            /* enable XKB autorepeat */
-            XkbSetDetectableAutoRepeat (backend_x11->xdpy,
-                                        True,
-                                        &detectable_autorepeat_supported);
+          backend_x11->have_xkb_autorepeat = detectable_autorepeat_supported;
 
-            backend_x11->have_xkb_autorepeat = detectable_autorepeat_supported;
-
-            CLUTTER_NOTE (BACKEND, "Detectable autorepeat: %s",
-                          backend_x11->have_xkb_autorepeat ? "supported"
-                                                           : "not supported");
-          }
-      }
-  }
-#endif /* HAVE_XKB */
+          CLUTTER_NOTE (BACKEND, "Detectable autorepeat: %s",
+                        backend_x11->have_xkb_autorepeat ? "supported"
+                                                         : "not supported");
+        }
+    }
 }
 
 static void
@@ -450,12 +431,10 @@ clutter_keymap_x11_finalize (GObject *gobject)
   keymap = CLUTTER_KEYMAP_X11 (gobject);
   translator = CLUTTER_EVENT_TRANSLATOR (keymap);
 
-#ifdef HAVE_XKB
   _clutter_backend_remove_event_translator (keymap->backend, translator);
 
   if (keymap->xkb_desc != NULL)
     XkbFreeKeyboard (keymap->xkb_desc, XkbAllComponentsMask, True);
-#endif
 
   G_OBJECT_CLASS (clutter_keymap_x11_parent_class)->finalize (gobject);
 }
@@ -503,7 +482,6 @@ clutter_keymap_x11_translate_event (ClutterEventTranslator *translator,
 
   retval = CLUTTER_TRANSLATE_CONTINUE;
 
-#ifdef HAVE_XKB
   if (xevent->type == keymap_x11->xkb_event_base)
     {
       XkbEvent *xkb_event = (XkbEvent *) xevent;
@@ -530,7 +508,6 @@ clutter_keymap_x11_translate_event (ClutterEventTranslator *translator,
           break;
         }
     }
-#endif /* HAVE_XKB */
 
   return retval;
 }
@@ -545,11 +522,7 @@ gint
 _clutter_keymap_x11_get_key_group (ClutterKeymapX11    *keymap,
                                    ClutterModifierType  state)
 {
-#ifdef HAVE_XKB
   return XkbGroupForCoreState (state);
-#else
-  return 0;
-#endif /* HAVE_XKB */
 }
 
 gboolean
@@ -606,7 +579,6 @@ _clutter_keymap_x11_translate_key_state (ClutterKeymapX11    *keymap,
 
   backend_x11 = CLUTTER_BACKEND_X11 (keymap->backend);
 
-#ifdef HAVE_XKB
   if (backend_x11->use_xkb)
     {
       XkbDescRec *xkb = get_xkb (keymap);
@@ -622,7 +594,6 @@ _clutter_keymap_x11_translate_key_state (ClutterKeymapX11    *keymap,
         retval = 0;
     }
   else
-#endif /* HAVE_XKB */
     retval = translate_keysym (keymap, hardware_keycode);
 
   if (mods_p)
@@ -644,7 +615,6 @@ _clutter_keymap_x11_get_is_modifier (ClutterKeymapX11 *keymap,
   if (keycode < keymap->min_keycode || keycode > keymap->max_keycode)
     return FALSE;
 
-#ifdef HAVE_XKB
   if (CLUTTER_BACKEND_X11 (keymap->backend)->use_xkb)
     {
       XkbDescRec *xkb = get_xkb (keymap);
@@ -652,7 +622,6 @@ _clutter_keymap_x11_get_is_modifier (ClutterKeymapX11 *keymap,
       if (xkb->map->modmap && xkb->map->modmap[keycode] != 0)
         return TRUE;
     }
-#endif /* HAVE_XKB */
 
   return FALSE;
 }
@@ -662,7 +631,6 @@ _clutter_keymap_x11_get_direction (ClutterKeymapX11 *keymap)
 {
   g_return_val_if_fail (CLUTTER_IS_KEYMAP_X11 (keymap), PANGO_DIRECTION_NEUTRAL);
 
-#ifdef HAVE_XKB
   if (CLUTTER_BACKEND_X11 (keymap->backend)->use_xkb)
     {
       if (!keymap->has_direction)
@@ -677,7 +645,6 @@ _clutter_keymap_x11_get_direction (ClutterKeymapX11 *keymap)
       return keymap->current_direction;
     }
   else
-#endif
     return PANGO_DIRECTION_NEUTRAL;
 }
 
@@ -687,7 +654,6 @@ clutter_keymap_x11_get_entries_for_keyval (ClutterKeymapX11  *keymap_x11,
                                            ClutterKeymapKey **keys,
                                            gint              *n_keys)
 {
-#ifdef HAVE_XKB
   if (CLUTTER_BACKEND_X11 (keymap_x11->backend)->use_xkb)
     {
       XkbDescRec *xkb = get_xkb (keymap_x11);
@@ -760,7 +726,6 @@ clutter_keymap_x11_get_entries_for_keyval (ClutterKeymapX11  *keymap_x11,
       return *n_keys > 0;
     }
   else
-#endif
     {
       return FALSE;
     }
@@ -771,7 +736,6 @@ clutter_keymap_x11_latch_modifiers (ClutterKeymapX11 *keymap_x11,
                                     uint32_t          level,
                                     gboolean          enable)
 {
-#ifdef HAVE_XKB
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (keymap_x11->backend);
   uint32_t modifiers[] = {
     0,
@@ -794,7 +758,6 @@ clutter_keymap_x11_latch_modifiers (ClutterKeymapX11 *keymap_x11,
   XkbLatchModifiers (clutter_x11_get_default_display (),
                      XkbUseCoreKbd, modifiers[level],
                      value);
-#endif
 }
 
 static uint32_t
