@@ -29,9 +29,7 @@
  *   Robert Bragg <robert@linux.intel.com>
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "cogl-config.h"
 
 #include <string.h>
 
@@ -40,11 +38,11 @@
 
 #include "cogl-display-private.h"
 #include "cogl-renderer-private.h"
-#include "cogl-winsys-private.h"
 #ifdef COGL_HAS_WAYLAND_EGL_SERVER_SUPPORT
 #include "cogl-wayland-server.h"
 #endif
 #include "cogl-gtype-private.h"
+#include "winsys/cogl-winsys-private.h"
 
 static void _cogl_display_free (CoglDisplay *display);
 
