@@ -43,16 +43,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-/* Guard C code in headers, while including them from C++ */
-#ifdef  __cplusplus
-#define COGL_BEGIN_DECLS  extern "C" {
-#define COGL_END_DECLS    }
-#else
-#define COGL_BEGIN_DECLS
-#define COGL_END_DECLS
-#endif
-
-COGL_BEGIN_DECLS
+G_BEGIN_DECLS
 
 /**
  * SECTION:cogl-types
@@ -73,20 +64,6 @@ COGL_BEGIN_DECLS
  */
 typedef int CoglBool;
 
-#if __GNUC__ >= 4
-#define COGL_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
-#else
-#define COGL_GNUC_NULL_TERMINATED
-#endif
-
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)) && \
-  !defined (COGL_COMPILATION)
-#define COGL_GNUC_DEPRECATED                       \
-  __attribute__((__deprecated__))
-#else
-#define COGL_GNUC_DEPRECATED
-#endif /* __GNUC__ */
-
 /* Some structures are meant to be opaque but they have public
    definitions because we want the size to be public so they can be
    allocated on the stack. This macro is used to ensure that users
@@ -148,15 +125,6 @@ cogl_handle_ref (CoglHandle handle);
 void
 cogl_handle_unref (CoglHandle handle);
 
-/**
- * CoglFuncPtr:
- *
- * The type used by cogl for function pointers, note that this type
- * is used as a generic catch-all cast for function pointers and the
- * actual arguments and return type may be different.
- */
-typedef void (* CoglFuncPtr) (void);
-
 /* We forward declare this in cogl-types to avoid circular dependencies
  * between cogl-matrix.h, cogl-euler.h and cogl-quaterion.h */
 typedef struct _CoglMatrix      CoglMatrix;
@@ -891,6 +859,6 @@ typedef enum {
   COGL_STEREO_RIGHT
 } CoglStereoMode;
 
-COGL_END_DECLS
+G_END_DECLS
 
 #endif /* __COGL_TYPES_H__ */
