@@ -33,7 +33,7 @@
 #ifndef __COGL_CLUTTER_H__
 #define __COGL_CLUTTER_H__
 
-COGL_BEGIN_DECLS
+G_BEGIN_DECLS
 
 #define cogl_clutter_check_extension cogl_clutter_check_extension_CLUTTER
 COGL_DEPRECATED
@@ -49,6 +49,6 @@ cogl_clutter_winsys_has_feature (CoglWinsysFeature feature);
 void
 cogl_onscreen_clutter_backend_set_size (int width, int height);
 
-COGL_END_DECLS
+G_END_DECLS
 
 #endif /* __COGL_CLUTTER_H__ */
