@@ -79,7 +79,7 @@ typedef struct _CoglWinsysVtable
 
   /* Required functions */
 
-  CoglFuncPtr
+  GCallback
   (*renderer_get_proc_address) (CoglRenderer *renderer,
                                 const char *name,
                                 CoglBool in_core);
@@ -122,9 +122,6 @@ typedef struct _CoglWinsysVtable
                                         const int *rectangles,
                                         int n_rectangles);
 
-  void
-  (*onscreen_update_swap_throttled) (CoglOnscreen *onscreen);
-
   void
   (*onscreen_set_visibility) (CoglOnscreen *onscreen,
                               CoglBool visibility);
