@@ -26,9 +26,11 @@
 
 #include "backends/meta-backend-private.h"
 #include "backends/meta-dbus-session-watcher.h"
+#include "backends/meta-remote-access-controller-private.h"
 #include "backends/meta-screen-cast-monitor-stream.h"
 #include "backends/meta-screen-cast-stream.h"
-#include "backends/meta-remote-access-controller-private.h"
+#include "backends/meta-screen-cast-window-stream.h"
+#include "core/display-private.h"
 
 #define META_SCREEN_CAST_SESSION_DBUS_PATH "/org/gnome/Mutter/ScreenCast/Session"
 
@@ -333,6 +335,15 @@ handle_record_window (MetaDBusScreenCastSession *skeleton,
                       GVariant                  *properties_variant)
 {
   MetaScreenCastSession *session = META_SCREEN_CAST_SESSION (skeleton);
+  GDBusInterfaceSkeleton *interface_skeleton;
+  GDBusConnection *connection;
+  MetaWindow *window;
+  GError *error = NULL;
+  MetaDisplay *display;
+  GVariant *window_id_variant = NULL;
+  MetaScreenCastWindowStream *window_stream;
+  MetaScreenCastStream *stream;
+  char *stream_path;
 
   if (!check_permission (session, invocation))
     {
@@ -342,9 +353,59 @@ handle_record_window (MetaDBusScreenCastSession *skeleton,
       return TRUE;
     }
 
-  g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
-                                         G_DBUS_ERROR_FAILED,
-                                         "Recording a window not yet supported");
+  if (properties_variant)
+    window_id_variant = g_variant_lookup_value (properties_variant,
+                                                "window-id",
+                                                G_VARIANT_TYPE ("t"));
+
+  display = meta_get_display ();
+  if (window_id_variant)
+    {
+      uint64_t window_id;
+
+      g_variant_get (window_id_variant, "t", &window_id);
+      window = meta_display_get_window_from_id (display, window_id);
+    }
+  else
+    {
+      window = meta_display_get_focus_window (display);
+    }
+
+  if (!window)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_FAILED,
+                                             "Window not found");
+      return TRUE;
+    }
+
+  interface_skeleton = G_DBUS_INTERFACE_SKELETON (skeleton);
+  connection = g_dbus_interface_skeleton_get_connection (interface_skeleton);
+
+  window_stream = meta_screen_cast_window_stream_new (connection,
+                                                      window,
+                                                      &error);
+  if (!window_stream)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_FAILED,
+                                             "Failed to record window: %s",
+                                             error->message);
+      g_error_free (error);
+      return TRUE;
+    }
+
+  stream = META_SCREEN_CAST_STREAM (window_stream);
+  stream_path = meta_screen_cast_stream_get_object_path (stream);
+
+  session->streams = g_list_append (session->streams, stream);
+
+  g_signal_connect (stream, "closed", G_CALLBACK (on_stream_closed), session);
+
+  meta_dbus_screen_cast_session_complete_record_window (skeleton,
+                                                        invocation,
+                                                        stream_path);
+
   return TRUE;
 }
 
