@@ -27,6 +27,7 @@
 #ifndef __META_SHAPED_TEXTURE_PRIVATE_H__
 #define __META_SHAPED_TEXTURE_PRIVATE_H__
 
+#include "backends/meta-monitor-manager-private.h"
 #include "meta/meta-shaped-texture.h"
 
 ClutterActor *meta_shaped_texture_new (void);
@@ -41,5 +42,7 @@ void meta_shaped_texture_set_fallback_size (MetaShapedTexture *stex,
                                             int                fallback_height);
 gboolean meta_shaped_texture_is_obscured (MetaShapedTexture *self);
 cairo_region_t * meta_shaped_texture_get_opaque_region (MetaShapedTexture *stex);
+void meta_shaped_texture_set_transform (MetaShapedTexture    *stex,
+                                        MetaMonitorTransform  transform);
 
 #endif
