@@ -24,13 +24,9 @@
 #include <cairo.h>
 #include <glib.h>
 
+#include "backends/meta-backend-types.h"
 #include "clutter/clutter.h"
-
-typedef enum _MetaRoundingStrategy
-{
-  META_ROUNDING_STRATEGY_SHRINK,
-  META_ROUNDING_STRATEGY_GROW,
-} MetaRoundingStrategy;
+#include "core/boxes-private.h"
 
 /**
  * MetaRegionIterator:
@@ -110,4 +106,9 @@ cairo_region_t * meta_make_border_region (cairo_region_t *region,
                                           int             y_amount,
                                           gboolean        flip);
 
+cairo_region_t * meta_region_transform (cairo_region_t       *region,
+                                        MetaMonitorTransform  transform,
+                                        int                   width,
+                                        int                   height);
+
 #endif /* __META_REGION_UTILS_H__ */
