@@ -24,6 +24,7 @@
 
 #include <glib-object.h>
 
+#include "backends/meta-backend-types.h"
 #include "meta/boxes.h"
 #include "meta/common.h"
 
@@ -39,6 +40,12 @@ typedef enum
   FIXED_DIRECTION_Y    = 1 << 1,
 } FixedDirections;
 
+typedef enum _MetaRoundingStrategy
+{
+  META_ROUNDING_STRATEGY_SHRINK,
+  META_ROUNDING_STRATEGY_GROW,
+} MetaRoundingStrategy;
+
 /* Output functions -- note that the output buffer had better be big enough:
  *   rect_to_string:   RECT_LENGTH
  *   region_to_string: (RECT_LENGTH+strlen(separator_string)) *
@@ -219,6 +226,11 @@ GList* meta_rectangle_find_nonintersected_monitor_edges (
 gboolean meta_rectangle_is_adjecent_to (MetaRectangle *rect,
                                         MetaRectangle *other);
 
+void meta_rectangle_scale_double (const MetaRectangle  *rect,
+                                  double                scale,
+                                  MetaRoundingStrategy  rounding_strategy,
+                                  MetaRectangle        *dest);
+
 static inline ClutterRect
 meta_rectangle_to_clutter_rect (MetaRectangle *rect)
 {
@@ -234,4 +246,10 @@ meta_rectangle_to_clutter_rect (MetaRectangle *rect)
   };
 }
 
+void meta_rectangle_transform (const MetaRectangle  *rect,
+                               MetaMonitorTransform  transform,
+                               int                   width,
+                               int                   height,
+                               MetaRectangle        *dest);
+
 #endif /* META_BOXES_PRIVATE_H */
