@@ -166,9 +166,6 @@ gboolean meta_prefs_get_workspaces_only_on_primary (void);
 int      meta_prefs_get_draggable_border_width (void);
 int      meta_prefs_get_drag_threshold (void);
 
-gboolean meta_prefs_get_ignore_request_hide_titlebar (void);
-void     meta_prefs_set_ignore_request_hide_titlebar (gboolean whether);
-
 /**
  * MetaKeyBindingAction:
  * @META_KEYBINDING_ACTION_NONE: FILLME
