@@ -43,6 +43,16 @@ typedef struct _MetaWaylandGtkSurface
   gulong configure_handler_id;
 } MetaWaylandGtkSurface;
 
+struct _MetaWaylandGtkShell
+{
+  GObject parent;
+
+  GList *shell_resources;
+  uint32_t capabilities;
+};
+
+G_DEFINE_TYPE (MetaWaylandGtkShell, meta_wayland_gtk_shell, G_TYPE_OBJECT)
+
 static void
 gtk_surface_destructor (struct wl_resource *resource)
 {
@@ -311,11 +321,19 @@ gtk_shell_set_startup_id (struct wl_client   *client,
                           struct wl_resource *resource,
                           const char         *startup_id)
 {
+  MetaStartupSequence *sequence;
   MetaDisplay *display;
 
   display = meta_get_display ();
-  meta_startup_notification_remove_sequence (display->startup_notification,
-                                             startup_id);
+
+  sequence = meta_startup_notification_lookup_sequence (display->startup_notification,
+                                                        startup_id);
+  if (sequence)
+    {
+      meta_startup_sequence_complete (sequence);
+      meta_startup_notification_remove_sequence (display->startup_notification,
+                                                 sequence);
+    }
 }
 
 static void
@@ -348,34 +366,105 @@ static const struct gtk_shell1_interface meta_wayland_gtk_shell_interface = {
   gtk_shell_system_bell,
 };
 
+static void
+gtk_shell_destructor (struct wl_resource *resource)
+{
+  MetaWaylandGtkShell *gtk_shell = wl_resource_get_user_data (resource);
+
+  gtk_shell->shell_resources = g_list_remove (gtk_shell->shell_resources,
+                                              resource);
+}
+
 static void
 bind_gtk_shell (struct wl_client *client,
                 void             *data,
                 guint32           version,
                 guint32           id)
 {
+  MetaWaylandGtkShell *gtk_shell = data;
   struct wl_resource *resource;
-  uint32_t capabilities = 0;
 
   resource = wl_resource_create (client, &gtk_shell1_interface, version, id);
   wl_resource_set_implementation (resource, &meta_wayland_gtk_shell_interface,
-                                  data, NULL);
+                                  data, gtk_shell_destructor);
+
+  gtk_shell->shell_resources = g_list_prepend (gtk_shell->shell_resources,
+                                               resource);
+
+  gtk_shell1_send_capabilities (resource, gtk_shell->capabilities);
+}
+
+static void
+meta_wayland_gtk_shell_init (MetaWaylandGtkShell *gtk_shell)
+{
+}
+
+static void
+meta_wayland_gtk_shell_class_init (MetaWaylandGtkShellClass *klass)
+{
+  quark_gtk_surface_data =
+    g_quark_from_static_string ("-meta-wayland-gtk-shell-surface-data");
+}
+
+static uint32_t
+calculate_capabilities (void)
+{
+  uint32_t capabilities = 0;
 
   if (!meta_prefs_get_show_fallback_app_menu ())
     capabilities = GTK_SHELL1_CAPABILITY_GLOBAL_APP_MENU;
 
-  gtk_shell1_send_capabilities (resource, capabilities);
+  return capabilities;
 }
 
-void
-meta_wayland_gtk_shell_init (MetaWaylandCompositor *compositor)
+static void
+prefs_changed (MetaPreference pref,
+               gpointer       user_data)
 {
-  quark_gtk_surface_data =
-    g_quark_from_static_string ("-meta-wayland-gtk-shell-surface-data");
+  MetaWaylandGtkShell *gtk_shell = user_data;
+  uint32_t new_capabilities;
+  GList *l;
+
+  if (pref != META_PREF_BUTTON_LAYOUT)
+    return;
+
+  new_capabilities = calculate_capabilities ();
+  if (gtk_shell->capabilities == new_capabilities)
+    return;
+  gtk_shell->capabilities = new_capabilities;
+
+  for (l = gtk_shell->shell_resources; l; l = l->next)
+    {
+      struct wl_resource *resource = l->data;
+
+      gtk_shell1_send_capabilities (resource, gtk_shell->capabilities);
+    }
+}
+
+static MetaWaylandGtkShell *
+meta_wayland_gtk_shell_new (MetaWaylandCompositor *compositor)
+{
+  MetaWaylandGtkShell *gtk_shell;
+
+  gtk_shell = g_object_new (META_TYPE_WAYLAND_GTK_SHELL, NULL);
 
   if (wl_global_create (compositor->wayland_display,
                         &gtk_shell1_interface,
                         META_GTK_SHELL1_VERSION,
-                        compositor, bind_gtk_shell) == NULL)
+                        gtk_shell, bind_gtk_shell) == NULL)
     g_error ("Failed to register a global gtk-shell object");
+
+  gtk_shell->capabilities = calculate_capabilities ();
+
+  meta_prefs_add_listener (prefs_changed, gtk_shell);
+
+  return gtk_shell;
+}
+
+void
+meta_wayland_init_gtk_shell (MetaWaylandCompositor *compositor)
+{
+  g_object_set_data_full (G_OBJECT (compositor), "-meta-wayland-gtk-shell",
+                          meta_wayland_gtk_shell_new (compositor),
+                          g_object_unref);
 }
