@@ -99,6 +99,8 @@ meta_x11_display_dispose (GObject *object)
 {
   MetaX11Display *x11_display = META_X11_DISPLAY (object);
 
+  meta_x11_startup_notification_release (x11_display);
+
   meta_prefs_remove_listener (prefs_changed_callback, x11_display);
 
   meta_x11_display_ungrab_keys (x11_display);
@@ -486,13 +488,9 @@ static void
 set_x11_bell_is_audible (MetaX11Display *x11_display,
                          gboolean is_audible)
 {
-#ifdef HAVE_LIBCANBERRA
   /* When we are playing sounds using libcanberra support, we handle the
    * bell whether its an audible bell or a visible bell */
   gboolean enable_system_bell = FALSE;
-#else
-  gboolean enable_system_bell = is_audible;
-#endif /* HAVE_LIBCANBERRA */
 
   XkbChangeEnabledControls (x11_display->xdisplay,
                             XkbUseCoreKbd,
@@ -596,6 +594,7 @@ set_supported_hint (MetaX11Display *x11_display)
 
     x11_display->atom__GTK_FRAME_EXTENTS,
     x11_display->atom__GTK_SHOW_WINDOW_MENU,
+    x11_display->atom__GTK_EDGE_CONSTRAINTS,
   };
 
   XChangeProperty (x11_display->xdisplay,
@@ -1352,6 +1351,8 @@ meta_x11_display_new (MetaDisplay *display, GError **error)
 
   set_x11_bell_is_audible (x11_display, meta_prefs_bell_is_audible ());
 
+  meta_x11_startup_notification_init (x11_display);
+
   return x11_display;
 }
 
